import base64
import json

from appium.webdriver.webelement import WebElement

class FlutterElement(WebElement):
    def __init__(self, driver, element_id):
        super(FlutterElement, self).__init__(
            driver, element_id, w3c=True
        )


class FlutterFinder(object):
    def by_ancestor(self, serialized_finder, matching, match_root=False):
        return self._by_ancestor_or_descendant(
            type_='Ancestor',
            serialized_finder=serialized_finder,
            matching=matching,
            match_root=match_root
        )

    def by_descendant(self, serialized_finder, matching, match_root=False):
        return self._by_ancestor_or_descendant(
            type_='Descendant',
            serialized_finder=serialized_finder,
            matching=matching,
            match_root=match_root
        )

    def by_semantics_label(self, label, isRegExp=False):
        return self._serialize(dict(
            finderType='BySemanticsLabel',
            isRegExp=isRegExp,
            label=label
        ))

    def by_tooltip(self, text):
        return self._serialize(dict(
            finderType='ByTooltipMessage',
            text=text
        ))

    def by_text(self, text):
        return self._serialize(dict(
            finderType='ByText',
            text=text
        ))

    def by_type(self, type_):
        return self._serialize(dict(
            finderType='ByType',
            type=type_
        ))

    def by_value_key(self, key):
        return self._serialize(dict(
            finderType='ByValueKey',
            keyValueString=key,
            keyValueType='String' if isinstance(key, str) else 'int'
        ))

    def page_back(self):
        return self._serialize(dict(
            finderType='PageBack'
        ))

    def _serialize(self, finder_dict):
        return base64.b64encode(json.dumps(finder_dict))

    def _by_ancestor_or_descendant(self, type_, serialized_finder, matching, match_root=False):
        param = dict(finderType=type_, matchRoot=match_root)

        try:
            finder = json.dumps(base64.b64decode(serialized_finder))
        except:
            finder = dict()
        for finder_key, finder_value in finder.items():
            param['of_{}'.format(finder_key)] = finder_value

        try:
            matching = json.dumps(base64.b64decode(matching))
        except:
            matching = dict()
        for matching_key, matching_value in matching.items():
            param['matching_{}'.format(matching_key)] = matching_value

        return self._serialize(param)



#!/usr/bin/env python

import io
import os

from setuptools import find_packages, setup

setup(
    name='Appium-Flutter-Finder',
    version='0.1.1',
    description='An extention of finder for Appium flutter',
    long_description=io.open(os.path.join(os.path.dirname('__file__'), 'README.md'), encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    keywords=[
        'appium',
        'flutter',
        'python client',
        'mobile automation'
    ],
    author='Kazuaki Matsuo',
    author_email='fly.49.89.over@gmail.com',
    url='https://github.com/truongsinh/appium-flutter-driver',
    packages=find_packages(include=['appium_flutter_finder*']),
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Environment :: Console',
        'Environment :: MacOS X',
        'Environment :: Win32 (MS Windows)',
        'Intended Audience :: Developers',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Software Development :: Testing'
    ],
    install_requires=['Appium-Python-Client >= 0.4']
)
